@@ -1,4 +1,4 @@
-# $Header: /home/cvsroot/NetZ3950/Z3950/Connection.pm,v 1.25 2004/03/31 12:28:01 mike Exp $
+# $Header: /home/cvsroot/NetZ3950/Z3950/Connection.pm,v 1.26 2004/05/06 13:19:39 mike Exp $
 
 package Net::Z3950::Connection;
 use IO::Handle;
@@ -15,6 +15,7 @@ Net::Z3950::Connection - Connection to a Z39.50 server, with request queue
 
 	$conn = new Net::Z3950::Connection($hostname, $port);
 	$rs = $conn->search('au=kernighan and su=unix');
+	$sr = $conn->scan('au=kernighan and su=unix', $term, $pos);
 	# or
 	$mgr = $conn->manager();
 	$conn = $mgr->wait();
@@ -318,6 +319,13 @@ sub _dispatch {
 	### Should handle piggy-backed records and NSDs
 	return $which;
 
+    } elsif ($apdu->isa('Net::Z3950::APDU::ScanResponse')) {
+        $this->{op} = Net::Z3950::Op::Scan;
+        $this->{scanResponse} = $apdu;
+        my $which = $apdu->referenceId();
+        defined $which or die "no reference Id in scan response";
+        return $which;
+
     } elsif ($apdu->isa('Net::Z3950::APDU::PresentResponse')) {
 	$this->{op} = Net::Z3950::Op::Get;
 	$this->{presentResponse} = $apdu;
@@ -498,7 +506,7 @@ information set to ``no diagnostic records supplied by server''.
 
 =cut
 
-# PRIVATE to the startSearch() method
+# PRIVATE to the startSearch() and startScan() methods
 my %_queryTypes = (
     prefix => Net::Z3950::QueryType::Prefix,
     ccl => Net::Z3950::QueryType::CCL,
@@ -556,6 +564,65 @@ sub startSearch {
 }
 
 
+=head2 startScan()
+
+	$conn->startScan($scan);
+	$conn->startScan(-prefix => '@attr 1=5 programming');
+	$conn->startScan('@attr 1=5 programming');
+
+Executes a scan against the Z39.50 server to which I<$conn> is
+connected.  The scan parameters are represented by a query which is
+analysed for the term itself and the access-point in which it should
+occur.  This query can be specified in the ways as for
+C<startSearch()>.
+
+=cut
+
+sub startScan {
+    my $this = shift();
+    my $query = shift();
+    my($type, $value);
+
+    ### Too much shared code with startSearch()
+    if (ref $query) {
+	### Huh?  We don't actually have a *::Query type!
+        $type = $query->type();
+        $value = $query->value();
+    } else {
+        # Must be either (-querytype querystring) or just querystring
+        if ($query =~ /^-/) {
+            ($type = $query) =~ s/^-//;
+            $value = shift();
+        } else {
+            $type = $this->option('querytype');
+            $value = $query;
+        }
+        $query = undef;
+    }
+
+    my $queryType = $_queryTypes{$type};
+    die "undefined query type '$type'" if !defined $queryType;
+
+    # Generate the SCAN request and queue it up for subsequent dispatch
+    my $errmsg = '';
+    my $sr = Net::Z3950::makeScanRequest("scan",
+                                         $this->option('databaseName'),
+                                         $this->option('stepSize'),
+                                         $this->option('numberOfTermsRequested'),
+					 $this->option('preferredPositionInResponse'),
+                                         $queryType,
+                                         $value,
+                                         $errmsg);
+    die "can't make scan request: $errmsg" if !defined $sr;
+
+    $this->_enqueue($sr);
+
+    # Callback for asynchronous notification
+    my $cb = shift();
+    $this->{refId2cb}->{'scan'} = $cb if defined $cb;
+}
+
+
 # Decode record-syntax strings into enumerators
 sub preferredRecordSyntax {
     my $this = shift();
@@ -574,7 +641,7 @@ sub preferredRecordSyntax {
 }
 
 
-# PRIVATE to the new() and startSearch() methods
+# PRIVATE to the new(), startSearch() and startScan() methods
 sub _enqueue {
     my $this = shift();
     my($msg) = @_;
@@ -617,6 +684,32 @@ sub search {
 }
 
 
+=head2 scan()
+
+    $sr = $conn->scan($scan);
+
+This method performs a blocking scan, returning a reference
+to the scan result generated by the server. It takes the same
+arguments as C<startScan()>
+
+The returned structure is a C<Net::Z3950::APDU::ScanResponse> which
+can be pulled apart by inspection.  That may not be the nicest
+possible interface.
+
+=cut
+
+sub scan {
+    my $this = shift();
+
+    $this->startScan(@_);
+    if (!$this->expect(Net::Z3950::Op::Scan, "scan")) {
+	return undef;
+    }
+
+    return $this->scanResponse();
+}
+
+
 # Private method, shared with ResultSet.pm but not available to client
 # code.  Used to implement synchronous operations on top of async
 # ones: waits for something to happen on $conn's manager, checks that
@@ -687,6 +780,11 @@ the C<resultSet()> method described below.
 One or more result-set records have become available.  They may be
 obtained via the C<record()> method of the appropriate result set.
 
+=item C<Net::Z3950::Op::Scan>
+
+A scan response was received.  It may be obtained via the
+C<scanResponse()> method described below.
+
 =back
 
 =cut
@@ -792,6 +890,28 @@ sub resultSet {
 }
 
 
+=head2 scanResponse()
+
+    if ($op == Net::Z3950::Op::Scan) {
+        $sr = $conn->scanResponse();
+    }
+
+When a connection is known to have received a scan response, the
+response may be accessed via the connection's C<scanResponse()>.
+
+The returned structure is a C<Net::Z3950::APDU::ScanResponse> which
+can be pulled apart by inspection.  That may not be the nicest
+possible interface.
+
+=cut
+
+sub scanResponse {
+    my $this = shift();
+    die "not scan response" if $this->op() != Net::Z3950::Op::Scan;
+    return $this->{scanResponse};
+}
+
+
 =head2 resultSets()
 
 	@rs = $conn->resultSets();
@@ -856,7 +976,7 @@ sub close {
     $this->{readWatcher}->cancel() if defined $this->{readWatcher};
     $this->{writeWatcher}->cancel() if defined $this->{writeWatcher};
 
-    # XXX for a V.3 connection, we should really send a closeRequest
+    # ### for a V.3 connection, we should really send a closeRequest
     # and await a closeResponse, but thats a lot of extra coding effort
     # for very little gain. A server that can't cope with an
     # abrupty-severed connection isn't going to last for long in the real
